import { createContext, FC, useEffect, useState } from "react";
import {
  HubConnection,
  HubConnectionBuilder,
  LogLevel,
} from "@microsoft/signalr";
import { asTodo, StatusState, Todo } from "@src/core/entities/todo";

interface SocketContextProps {
  conn: HubConnection;
  todos: Todo[];
  inboxTodos: Todo[];
  getTodos: () => void;
  getInboxTodos: () => void;
  createTodo: (todoName: string, project: string) => void;
  updateTodo: (todoId: string, todoStatus: StatusState) => void;
  getTodoById(todoId: string): void;
  replaceTodo(todo: Todo): void;
}

export const SocketContext = createContext<SocketContextProps>({
  conn: void 0,
  todos: [],
  inboxTodos: [],
  getTodos: () => {},
  getInboxTodos: () => {},
  createTodo: (todoName, project) => {},
  updateTodo: (todoId, todoStatus) => {},
  getTodoById(todoId: string) {},
  replaceTodo(todo: Todo) {},
});

export const SocketProvider: FC = (props) => {
  const [conn, setConn] = useState<HubConnection>();
  const [todos, setTodos] = useState<Todo[]>([]);
  const [inboxTodos, setInboxTodos] = useState<Todo[]>([]);

  useEffect(() => {
    const serverUrl =
      process.env.NEXT_PUBLIC_SERVER_URL || "http://localhost:5000";

    const connection = new HubConnectionBuilder()
      .withUrl(`${serverUrl}/hubs/todo`)
      .withAutomaticReconnect()
      .configureLogging(LogLevel.Information)
      .build();

    connection.on("getTodos", (rawTodos) => {
      const newTodos = JSON.parse(rawTodos) as Todo[];
      const validatedTodos = newTodos.map(asTodo);
      setTodos(validatedTodos);
    });

    connection.on("getInboxTodos", (rawTodos) => {
      const newTodos = JSON.parse(rawTodos) as Todo[];
      const validatedTodos = newTodos
        .map(asTodo)
        .filter((t) => t.status == false);
      setInboxTodos(validatedTodos);
    });

    connection.on("getTodo", (todo) => {
      const newTodo = JSON.parse(todo) as Todo;
      const validatedTodo = asTodo(newTodo);
      setTodos([
        ...todos.filter((t) => t.id !== validatedTodo.id),
        validatedTodo,
      ]);
    });

    connection.start().then(() => {
      setConn(connection);
    }).catch(e => {
        window.location.href = `${serverUrl}/api/auth/login?returnUrl=${window.location.href}`
    });
  }, []);

  return (
    <SocketContext.Provider
      value={{
        conn,
        todos,
        inboxTodos,
        getTodos: () => {
          conn.invoke("GetTodos").catch(console.error);
        },
        getInboxTodos: () => {
          conn.invoke("GetInboxTodos").catch(console.error);
        },
        createTodo: (todoName, project) => {
          conn.invoke("CreateTodo", todoName, project).catch(console.error);
        },
        updateTodo: (todoId, todoStatus) => {
          conn.invoke("UpdateTodo", todoId, todoStatus).catch(console.error);
        },
        getTodoById(todoId: string) {
          conn.invoke("GetTodo", todoId).catch(console.error);
        },
        replaceTodo: (todo) => {
          conn.invoke("ReplaceTodo", JSON.stringify(todo)).catch(console.error);
        },
      }}
    >
      {conn ? props.children : "loading"}
    </SocketContext.Provider>
  );
};

export default SocketProvider;
